package queueIncomplete;

import static lib.Rand.*;
import static lib.IO.*;

public class WaitInLine {
	
	private static String[] firstNames = {"Christian", "Sebastian", "Michael", "Stefan", "Jan", "Julia", "Katrin", "Stefanie", "Melanie", "Sandra"};
	private static String[] lastNames = {"Keckeis", "Seeber", "Kothmiller", "Pichler"};

	public static void main(String[] args) {
		Queue q = new Queue();
		
		// Program does never end
		do {
			serve(q);
			
			leave(q);
			
			stepIn(q);
			
			// Who is currently waiting?
			println("\nCurrently waiting people: " + q.size());
			println(q);
			try {
				// Wait a little
				Thread.sleep(5000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		} while (true);
	}
	
	/**
	 * A new person steps in the door and waits in line.
	 */
	private static void stepIn(Queue q) {
		// Randomly determine whether a person steps in or not (p=2/3)
		if (getRandomInt(0, 3) == 0)
			// No person is enqueued
			return;
			
		// Create person
		String firstName = firstNames[getRandomInt(0, firstNames.length)];
		String lastName = lastNames[getRandomInt(0, lastNames.length)];
		Person person = new Person(firstName, lastName);
		
		println(person +" steps in and waits in line.");
		q.enqueue(person);
	}
	
	/**
	 * Person is served. It is randomly determined whether the person
	 * leaves now or not.
	 * @param q Queue
	 */
	private static void serve(Queue q) {
		// Randomly determine whether a person is finished or not (p=1/2)
		if (getRandomInt(0, 2) == 0) {
			// Person leaves
			Person person = q.dequeue();
			if (person != null)
				println(person + " was served and leaves the shop.");
		}
	}
	
	/**
	 * Sometimes a person is not willing to wait any longer and leaves.
	 * @param q Queue
	 */
	private static void leave(Queue q) {
		// Randomly determine whether a person wants to leave or not (p=1/4)
		if (getRandomInt(0, 5) == 0) {
			
			// Person leaves
			Person person = q.leave(getRandomInt(0, q.size()));
			if (person != null)
				println(person + " is tired of waiting and leaves the shop.");
		}
	}

}
